package com.ejgv.gvi.dao;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.AsyncTask;

import com.ejgv.gvi.dao.dto.DTOMessage;
import com.ejgv.gvi.dao.dto.DTOPnsMsgOperation;
import com.ejgv.gvi.dao.dto.DTOResponseGetMessages;
import com.ejgv.gvi.dao.dto.DTOResponseMsgPendingOperations;
import com.ejgv.gvi.dao.dto.DTOResponseTagList;
import com.ejgv.gvi.restapipns.dto.DTOPnsMessage;
import com.ejgv.gvi.restapipns.dto.DTOPnsMsgPrivateResponseValues;
import com.ejgv.gvi.restapipns.dto.DTOResponseSimple;

/**
 * Created by koldo on 22/12/15.
 */
public class DAOService
{
    private static final String TAG = "DAOService";

    interface DAOMessageInterface
    {
        long insert(DTOPnsMessage pnsMessage) throws Exception;
        long[] insert(DTOPnsMessage[] pnsMessages) throws Exception;
        long updateTsRead(String trans_id) throws Exception;
        long delete() throws Exception;
        long delete(String trans_id) throws Exception;
        long delete(String[] trans_ids) throws Exception;
        long deleteOver(int limit) throws Exception;
        DTOMessage[] selectMessages() throws Exception;
        DTOMessage[] selectMessagesFrom(String trans_id, long ts) throws Exception;
        DTOMessage[] selectMessagesTagsLike(String query) throws Exception;
    }

    interface DAOMsgPendingOperationInterface
    {
        long insert(DTOPnsMsgOperation msgOperation) throws Exception;
        long[] insertOperations(char operation, String[] trans_ids) throws Exception;
        long delete() throws Exception;
        long deleteOperation(DTOPnsMsgOperation msgOperation) throws Exception;
        long deleteOperations(char operation, String[] trans_ids) throws Exception;
        DTOPnsMsgOperation[] selectPendingOperations() throws Exception;
    }


    interface DAOMsgResponseValueInterface
    {
        long insert(String trans_id, DTOPnsMsgPrivateResponseValues value) throws Exception;
        long[] insert(String trans_id, DTOPnsMsgPrivateResponseValues[] values) throws Exception;
        long delete() throws Exception;
        long delete(String trans_id, String code) throws Exception;
    }

    interface DAOMsgResponsedInterface
    {
        long insert(String trans_id, String responseCode, long tsResponse) throws Exception;
        long update(String trans_id, String responseCode) throws Exception;
        long delete() throws Exception;
        long delete(String trans_id) throws Exception;
    }

    interface DAOVirtualTagsInterface
    {
        void populate(String languageName) throws Exception;
        long delete() throws Exception;
        /*long delete(String tag) throws Exception;*/
        String[] selectTags() throws Exception;
        String[] selectTagsMatch(String query) throws Exception;
    }

    interface DAOJoinInterface
    {
        DTOMessage selectPrivateMessage(String trans_id);
        DTOMessage selectMsgResponseValues(String trans_id) throws Exception;
    }

    public static final String RESPONSE_RESULT_OK = "ok";
    public static final String RESPONSE_RESULT_ERROR = "error";

    static final String DDBB_ERROR_CODE = "DB.";
    public static final int DDBB_ERROR_EMPTY_DAO = 1;
    public static final int DDBB_ERROR = 99;

    private Context mContext;
    private SQLiteOpenHelper mDatabaseHelper;

    public DAOService(Context context)
    {
        mContext = context;
        mDatabaseHelper = new DDBBHelper(context.getApplicationContext());
    }

    /**
     * Opening writable database
     *
     * @return database
     * @throws Exception
     */
    private SQLiteDatabase openDBToWrite() throws Exception
    {
        return mDatabaseHelper.getWritableDatabase();
    }

    /**
     * opening readable database
     *
     * @return database
     * @throws Exception
     */
    private SQLiteDatabase openDBToRead() throws Exception
    {
        return mDatabaseHelper.getReadableDatabase();
    }

    /**
     * The class that uses createPopulateTagsCall method must implements this interface
     * to interact with createPopulateTagsCall
     */
    public interface OnPopulateTagsListener
    {
        void onPostPopulateTagsAsyncTask(DTOResponseSimple response);
    }
    /**
     * Create PopulateTagsAsyncTask instance
     *
     * @param listener
     * @return AsyncTask
     */
    public AsyncTask createPopulateTagsCall(OnPopulateTagsListener listener,
                                            String languageName)
    {
        return new PopulateTagsAsyncTask(mDatabaseHelper, listener,
                                        languageName);
    }

    /**
     * The class that uses createGetTagsCall method must implements this interface
     * to interact with GetTagsAsyncTask
     */
    public interface OnGetTagsListener
    {
        void onPostGetTagsAsyncTask(DTOResponseTagList response);
    }
    /**
     * Create GetTagsAsyncTask instance
     *
     * @param listener
     * @return AsyncTask
     */
    public AsyncTask createGetTagsCall(OnGetTagsListener listener)
    {
        return new GetTagsAsyncTask(mDatabaseHelper, listener);
    }

    /**
     * The class that uses createGetMessagesCall method must implements this interface
     * to interact with GetMessagesAsyncTask
     */
    public interface OnGetMessagesListener
    {
        void onPostGetMessagesAsyncTask(DTOResponseGetMessages response);
    }
    /**
     * Create GetMessagesAsyncTask instance
     *
     * @param listener
     * @return AsyncTask
     */
    public AsyncTask createGetMessagesCall(OnGetMessagesListener listener)
    {
        return new GetMessagesAsyncTask(mDatabaseHelper, listener);
    }
    /**
     * Create GetMessagesAsyncTask instance
     *
     * @param listener
     * @return AsyncTask
     */
    public AsyncTask createGetMessagesAfterDeleteOnesOverLimitCall(OnGetMessagesListener listener, int messagesLimit )
    {
        return GetMessagesAsyncTask.createGetMessagesAfterDeleteOnesOverLimit(mDatabaseHelper, listener, messagesLimit);
    }

    /**
     * The class that uses createGetMessagesFromCall method must implements this interface
     * to interact with GetMessagesFromAsyncTask
     */
    public interface OnGetMessagesFromListener
    {
        void onPostGetMessagesFromAsyncTask(DTOResponseGetMessages response);
    }
    /**
     * Create GetMessagesFromAsyncTask instance
     *
     * @param listener
     * @param trans_id
     * @param ts
     * @return
     */
    public AsyncTask createGetMessagesFromCall(OnGetMessagesFromListener listener,
                                               String trans_id, long ts)
    {
        return new GetMessagesFromAsyncTask(mDatabaseHelper, listener,
                                            trans_id, ts);
    }

    /**
     * The class that uses createGetPrivateMessageCall method must implements this interface
     * to interact with GetPrivateMessageAsyncTask
     */
    public interface OnGetPrivateMessageListener
    {
        void onPostGetPrivateMessageAsyncTask(DTOResponseGetMessages response);
    }
    /**
     * Create GetPrivateMessageAsyncTask instance
     *
     * @param listener
     * @param trans_id
     * @return
     */
    public AsyncTask createGetPrivateMessageCall(OnGetPrivateMessageListener listener,
                                               String trans_id)
    {
        return new GetPrivateMessageAsyncTask(mDatabaseHelper, listener,
                                                trans_id);
    }

    /**
     * Delete messages call
     *
     * @param trans_id
     */
    public Thread createMarkAsReadCall(String trans_id)
    {
        return new Thread(new MarkAsReadRunnable(mDatabaseHelper, trans_id));
    }

    /**
     * The class that uses createAddMessagesCall method must implements this interface
     * to interact with AddMessagesAsyncTask
     */
    public interface OnAddMessagesListener
    {
        void onPostAddMessagesAsyncTask(DTOResponseGetMessages response);
    }
    /**
     * Create AddMessagesAsyncTask instance
     *
     * @param listener
     * @param pnsMessages
     * @return AsyncTask
     */
    public AsyncTask createAddMessagesCall(OnAddMessagesListener listener,
                                           DTOPnsMessage[] pnsMessages, int messagesMaxLimit)
    {
        return new AddMessagesAsyncTask(mDatabaseHelper, listener,
                                        pnsMessages, messagesMaxLimit);
    }

    /**
     * Delete messages call
     *
     * @param trans_ids
     */
    public Thread createDeleteMessagesCall(String[] trans_ids)
    {
        return new Thread(new DeleteMessagesRunnable(mDatabaseHelper, trans_ids));
    }

    /**
     * Delete messages over a limit
     *
     * @param messagesLimit
     */
    public Thread createDeleteMessagesOverLimitCall(int messagesLimit)
    {
        return new Thread(new DeleteMessagesOverLimitRunnable(mDatabaseHelper, messagesLimit));
    }

    /**
     * The class that uses createGetMsgPendingOperationsCall method must implements this interface
     * to interact with GetMsgPendingOperationsAsyncTask
     */
    public interface OnGetMsgPendingOperationsListener
    {
        void onPostGetMsgPendingOperationsAsyncTask(DTOResponseMsgPendingOperations response);
    }
    /**
     * Create GetMsgPendingOperationsAsyncTask instance
     *
     * @param listener
     * @return AsyncTask
     */
    public AsyncTask createGetMsgPendingOperationsCall(OnGetMsgPendingOperationsListener listener)
    {
        return new GetMsgPendingOperationsAsyncTask(mDatabaseHelper, listener);
    }

    /**
     * Delete pending same operations call
     *
     * @param operation
     * @param trans_ids
     * @return
     */
    public Thread createDeleteMsgPendingOperationsCall(char operation, String[] trans_ids)
    {
        return new Thread(new DeleteMsgPendingOperationsRunnable(mDatabaseHelper, operation, trans_ids));
    }

    /**
     * The class that uses createGetMsgResponseValuesCall method must implements this interface
     * to interact with GetMsgResponseValuesAsyncTask
     */
    public interface OnGetMsgResponseValuesListener
    {
        void onPostGetMsgResponseValuesAsyncTask(DTOResponseGetMessages response);
    }
    /**
     * Create GetMsgResponseValuesAsyncTask instance
     *
     * @param listener
     * @param trans_id
     * @return AsyncTask
     */
    public AsyncTask createGetMsgResponseValuesCall(OnGetMsgResponseValuesListener listener,
                                                    String trans_id)
    {
        return new GetMsgResponseValuesAsyncTask(mDatabaseHelper, listener,
                                                trans_id);
    }

    /**
     * Add message responsed code
     * @param trans_id
     * @param responsedCode
     * @param tsResponse
     * @return
     */
    public Thread createAddMessageResponsedCall(String trans_id, String responsedCode, long tsResponse)
    {
        return new Thread(new AddMessageResponsedRunnable(mDatabaseHelper,
                                                        trans_id, responsedCode, tsResponse));
    }
}